mischief-ecs
Safe HaskellNone
LanguageGHC2024

Mischief.ECS.Components.Spawn

Synopsis

Documentation

getOrAddPairId :: Pair -> System ComponentId Source #

Get the id of a component - entity pair. In case the component isn't registered, it will give it a new id.

getOrAddComponentId :: ComponentType -> System ComponentId Source #

Get the id of a component. In case the component isn't registered, it will give it a new id.

tryMeta :: forall c m w. (Component c, MonadSystem w m) => m (Maybe Entity) Source #

getHook :: forall {k} (c :: k). Hook c -> HookContext -> System () Source #

getHookRel :: forall {k} (c :: k). HookRel c -> HookContextRel -> System () Source #