| Safe Haskell | None |
|---|---|
| Language | GHC2024 |
Mischief.ECS.Systems
Description
Module with utility functions for creating and scheduling systems.
Documentation
add :: (HasCallStack, Schedule sc, SystemConfig s) => sc -> s -> System () Source #
| Safe Haskell | None |
|---|---|
| Language | GHC2024 |
Mischief.ECS.Systems
Description
Module with utility functions for creating and scheduling systems.
add :: (HasCallStack, Schedule sc, SystemConfig s) => sc -> s -> System () Source #