mischief-ecs
Safe HaskellNone
LanguageGHC2024

Mischief.ECS.Tables

Synopsis

Documentation

newtype Tables Source #

Constructors

Tables 

Fields

newtype Column Source #

Constructors

Column (IOVec ComponentData) 

tapMap :: (Monad m, Ord k) => Map k a -> k -> (a -> m ()) -> m () Source #

runs the specified monadic action with the value of the given map key as input, if the key exists. if it doesn't, do nothing

replaceComponentsIntoMap Source #

Arguments

:: ProcessedBundleData 
-> Maybe Tick

The current tick that will be as the components' change tick.

-> EntityPointer 
-> Map ComponentId Column 
-> IO () 

replaceComponentsIntoTable Source #

Arguments

:: ProcessedBundleData 
-> Maybe Tick

The current tick that will be as the components' change tick.

-> EntityPointer 
-> Table 
-> IO ()