Mischief.ECS.Resources
insertRes :: (Component r, Bundle r) => r -> System () Source #
Insert a resource into the World. If the resource already exists, its value will be overwritten.
res :: Component c => System (Maybe c) Source #
Get the value of a resource from the World.
resOrInsert :: (Component r, Bundle r) => r -> System r Source #
Get the value of a resource, or insert a value if it doesn't exist.