| Safe Haskell | None |
|---|---|
| Language | GHC2024 |
Mischief.ECS.World.Insert
Synopsis
- data Exception' = Exception'
- insert :: HasCallStack => forall b. Bundle b => b -> Entity -> System ()
- insertNew :: Bundle b => b -> Entity -> System ()
- insertIfNeq :: BundleEq b => b -> Entity -> System ()
- triggerAddEvent :: ProcessedBundleData -> Entity -> System ()
- triggerAddEventC :: ErasedComponent -> Entity -> System ()
- triggerAddEventR :: ErasedComponent -> Entity -> Entity -> System ()
- triggerSetEvent :: ProcessedBundleData -> Entity -> System ()
- triggerSetEventC :: ErasedComponent -> Entity -> System ()
- triggerSetEventR :: ErasedComponent -> Entity -> Entity -> System ()
Documentation
data Exception' Source #
Constructors
| Exception' |
Instances
| Exception Exception' Source # | |
Defined in Mischief.ECS.World.Insert Methods toException :: Exception' -> SomeException # fromException :: SomeException -> Maybe Exception' # displayException :: Exception' -> String # backtraceDesired :: Exception' -> Bool # | |
| Show Exception' Source # | |
Defined in Mischief.ECS.World.Insert Methods showsPrec :: Int -> Exception' -> ShowS # show :: Exception' -> String # showList :: [Exception'] -> ShowS # | |
insert :: HasCallStack => forall b. Bundle b => b -> Entity -> System () Source #
Insert a bundle of components on an Entity.
If the entity already contains these components, their values will be updated in-place instead of causing an archetype change.
insertNew :: Bundle b => b -> Entity -> System () Source #
Insert a bundle of components on an Entity.
Only the components that the entity doesn't already have will be inserted, and the rest ignored.
triggerAddEvent :: ProcessedBundleData -> Entity -> System () Source #
triggerAddEventC :: ErasedComponent -> Entity -> System () Source #
triggerAddEventR :: ErasedComponent -> Entity -> Entity -> System () Source #
triggerSetEvent :: ProcessedBundleData -> Entity -> System () Source #
triggerSetEventC :: ErasedComponent -> Entity -> System () Source #
triggerSetEventR :: ErasedComponent -> Entity -> Entity -> System () Source #