mischief-ecs
Safe HaskellNone
LanguageGHC2024

Mischief.ECS.World.Utils

Synopsis

Documentation

setChangedTickOfComponents :: ProcessedBundleData -> (ComponentId -> Bool) -> Tick -> ProcessedBundleData Source #

Sets the change tick of certain elements of the bundle to the specified Tick.

setAddedTickOfComponents :: ProcessedBundleData -> (ComponentId -> Bool) -> Tick -> ProcessedBundleData Source #

Sets the added tick of certain elements of the bundle to the specified Tick.

isAlive :: forall m w. MonadSystem w m => Entity -> m Bool Source #

expect :: HasCallStack => forall (m :: Type -> Type) w a. MonadSystem w m => Text -> Maybe a -> m a Source #

newtype GetSystem Source #

Constructors

GetSystem (forall c. Component c => Proxy c -> Entity -> System (Maybe c))